#include <active_check.h>
#include <board_def.h>
#include <eth_init.h>

#define GPIO_ETH_POWER  xcat(GPIO, ETH_POWER_GPIO_N)
#define RCC_ETH_POWER   xcat(RCC_AHB1Periph_GPIO, ETH_POWER_GPIO_N)
#define PIN_ETH_POWER   xcat(GPIO_Pin_, ETH_POWER_PIN_N)
#define ETH_POWER_OFF   (!(ETH_POWER_ON))

static int ETHinterfaceMIIconfigure(void);
static int ETHinterfaceRMIIconfigure(void);
static void ETHpowerConfigure(void);

int ETHconfigureInterface() {
  static const int maxAttempts = 1000000;
  int result;

  if (ETH_PHY_INTERFACE == ETH_MII)
    result = ETHinterfaceMIIconfigure();
  else if (ETH_PHY_INTERFACE == ETH_RMII)
    result = ETHinterfaceRMIIconfigure();
  else
    result = -1;
  if (result < 0)
    return -1;
  ETHpowerConfigure();
  ETHpowerUp();
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_ETH_MAC |
                         RCC_AHB1Periph_ETH_MAC_Tx |
                         RCC_AHB1Periph_ETH_MAC_Rx,
                         ENABLE);
  ETH_DeInit();
  ETH_SoftwareReset();
  active_check(ETH_GetSoftwareResetStatus() == RESET, maxAttempts);
  return 0;
}

int ETHinterfaceMIIconfigure() {
  GPIO_InitTypeDef GPIO_InitStruct;

  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA | RCC_AHB1Periph_GPIOB |
                         RCC_AHB1Periph_GPIOC | RCC_AHB1Periph_GPIOG |
                         RCC_AHB1Periph_GPIOH | RCC_AHB1Periph_GPIOI,
                         ENABLE);
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);

  GPIO_InitStruct.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL ;

  if (ETH_PHY_CLOCK == ETH_CLOCK_AUTONOMOUS) {
  }
  else if (ETH_PHY_CLOCK == ETH_CLOCK_PROVIDED_BY_MICRO &&
           HSE_VALUE == 25000000) {
    /* Output HSE clock (25 MHz) on MCO1 pin (PA8) to clock the PHY */
    GPIO_InitStruct.GPIO_Pin = GPIO_Pin_8;
    GPIO_Init(GPIOA, &GPIO_InitStruct);
    RCC_MCO1Config(RCC_MCO1Source_HSE, RCC_MCO1Div_1);
  }
  else {
    return -1;
  }

  SYSCFG_ETH_MediaInterfaceConfig(SYSCFG_ETH_MediaInterface_MII);

  /* Ethernet pins configuration
      ETH_MDIO -------------------------> PA2
      ETH_MDC --------------------------> PC1
      ETH_PPS_OUT ----------------------> PB5 (not used)
      ETH_MII_CRS ----------------------> PH2
      ETH_MII_COL ----------------------> PH3
      ETH_MII_RX_ER --------------------> PI10
      ETH_MII_RXD2 ---------------------> PH6
      ETH_MII_RXD3 ---------------------> PH7
      ETH_MII_TX_CLK -------------------> PC3
      ETH_MII_TXD2 ---------------------> PC2
      ETH_MII_TXD3 ---------------------> PB8
      ETH_MII_RX_CLK/ETH_RMII_REF_CLK---> PA1
      ETH_MII_RX_DV/ETH_RMII_CRS_DV ----> PA7
      ETH_MII_RXD0/ETH_RMII_RXD0 -------> PC4
      ETH_MII_RXD1/ETH_RMII_RXD1 -------> PC5
      ETH_MII_TX_EN/ETH_RMII_TX_EN -----> PG11
      ETH_MII_TXD0/ETH_RMII_TXD0 -------> PG13
      ETH_MII_TXD1/ETH_RMII_TXD1 -------> PG14
  */

  GPIO_InitStruct.GPIO_Pin = GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_7;
  GPIO_Init(GPIOA, &GPIO_InitStruct);
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource1, GPIO_AF_ETH);
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource2, GPIO_AF_ETH);
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource7, GPIO_AF_ETH);

  GPIO_InitStruct.GPIO_Pin = GPIO_Pin_8;
  GPIO_Init(GPIOB, &GPIO_InitStruct);
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource8, GPIO_AF_ETH);

  GPIO_InitStruct.GPIO_Pin = GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 |
                             GPIO_Pin_4 | GPIO_Pin_5;
  GPIO_Init(GPIOC, &GPIO_InitStruct);
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource1, GPIO_AF_ETH);
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource2, GPIO_AF_ETH);
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource3, GPIO_AF_ETH);
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource4, GPIO_AF_ETH);
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource5, GPIO_AF_ETH);

  GPIO_InitStruct.GPIO_Pin =  GPIO_Pin_11 | GPIO_Pin_13 | GPIO_Pin_14;
  GPIO_Init(GPIOG, &GPIO_InitStruct);
  GPIO_PinAFConfig(GPIOG, GPIO_PinSource11, GPIO_AF_ETH);
  GPIO_PinAFConfig(GPIOG, GPIO_PinSource13, GPIO_AF_ETH);
  GPIO_PinAFConfig(GPIOG, GPIO_PinSource14, GPIO_AF_ETH);

  GPIO_InitStruct.GPIO_Pin = GPIO_Pin_2 | GPIO_Pin_3 |
                             GPIO_Pin_6 | GPIO_Pin_7;
  GPIO_Init(GPIOH, &GPIO_InitStruct);
  GPIO_PinAFConfig(GPIOH, GPIO_PinSource2, GPIO_AF_ETH);
  GPIO_PinAFConfig(GPIOH, GPIO_PinSource3, GPIO_AF_ETH);
  GPIO_PinAFConfig(GPIOH, GPIO_PinSource6, GPIO_AF_ETH);
  GPIO_PinAFConfig(GPIOH, GPIO_PinSource7, GPIO_AF_ETH);

  GPIO_InitStruct.GPIO_Pin = GPIO_Pin_10;
  GPIO_Init(GPIOI, &GPIO_InitStruct);
  GPIO_PinAFConfig(GPIOI, GPIO_PinSource10, GPIO_AF_ETH);

  return 0;
}

int ETHinterfaceRMIIconfigure() {
  GPIO_InitTypeDef GPIO_InitStruct;

  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA | RCC_AHB1Periph_GPIOC |
                         RCC_AHB1Periph_GPIOG, ENABLE);
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);

  GPIO_InitStruct.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL ;

  if (ETH_PHY_CLOCK == ETH_CLOCK_AUTONOMOUS) {
  }
  #if 0
  else if (ETH_PHY_CLOCK == ETH_CLOCK_PROVIDED_BY_MICRO) {
    /* Output 50 MHz on MCO1 pin (PA8) to clock the PHY.
       TODO: PLLCLK must be 100 MHz.*/
    GPIO_InitStruct.GPIO_Pin = GPIO_Pin_8;
    GPIO_Init(GPIOA, &GPIO_InitStruct);
    RCC_MCO1Config(RCC_MCO1Source_PLLCLK, RCC_MCO1Div_2);
  }
  #endif
  else {
    return -1;
  }

  SYSCFG_ETH_MediaInterfaceConfig(SYSCFG_ETH_MediaInterface_RMII);

  /* Ethernet pins configuration
      ETH_MDIO ------------> PA2
      ETH_MDC -------------> PC1
      ETH_RMII_REF_CLK-----> PA1
      ETH_RMII_CRS_DV -----> PA7
      ETH_RMII_RXD0 -------> PC4
      ETH_RMII_RXD1 -------> PC5
      ETH_RMII_TX_EN ------> PG11
      ETH_RMII_TXD0 -------> PG13
      ETH_RMII_TXD1 -------> PG14
  */

  GPIO_InitStruct.GPIO_Pin = GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_7;
  GPIO_Init(GPIOA, &GPIO_InitStruct);
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource1, GPIO_AF_ETH);
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource2, GPIO_AF_ETH);
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource7, GPIO_AF_ETH);

  GPIO_InitStruct.GPIO_Pin = GPIO_Pin_1 | GPIO_Pin_4 | GPIO_Pin_5;
  GPIO_Init(GPIOC, &GPIO_InitStruct);
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource1, GPIO_AF_ETH);
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource4, GPIO_AF_ETH);
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource5, GPIO_AF_ETH);

  GPIO_InitStruct.GPIO_Pin =  GPIO_Pin_11 | GPIO_Pin_13 | GPIO_Pin_14;
  GPIO_Init(GPIOG, &GPIO_InitStruct);
  GPIO_PinAFConfig(GPIOG, GPIO_PinSource11, GPIO_AF_ETH);
  GPIO_PinAFConfig(GPIOG, GPIO_PinSource13, GPIO_AF_ETH);
  GPIO_PinAFConfig(GPIOG, GPIO_PinSource14, GPIO_AF_ETH);

  return 0;
}

/* Konfiguruj wyprowadzenie PWRDWN modułu ethernetowego (PHY). */
void ETHpowerConfigure() {
  GPIO_InitTypeDef GPIO_InitStruct;

  RCC_AHB1PeriphClockCmd(RCC_ETH_POWER, ENABLE);
  GPIO_InitStruct.GPIO_Pin = PIN_ETH_POWER;
  GPIO_InitStruct.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_InitStruct.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStruct.GPIO_OType = GPIO_OType_OD;
  GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL ;
  GPIO_Init(GPIO_ETH_POWER, &GPIO_InitStruct);
}

/* Wyłącz zasilanie modułu ethernetowego. */
void ETHpowerDown() {
  /* ETH_PowerDownCmd(ENABLE); */
  GPIO_WriteBit(GPIO_ETH_POWER, PIN_ETH_POWER, ETH_POWER_OFF);
}

/* Włącz zasilanie modułu ethernetowego.
   Po wyzerowaniu wyjście jest w stanie niskim. */
void ETHpowerUp() {
  GPIO_WriteBit(GPIO_ETH_POWER, PIN_ETH_POWER, ETH_POWER_ON);
}
